% This function takes as input an input waveform (as a column vector),
% and the DFT length (in samples).  It performs Hanning-windowed DFTs
% on the input using a window step of one quarter the window length.
%
% NOTE: I have not tested this code in any cases where N is not a
% power of 2.
%
% NOTE: Normally this will be used with real-valued signals, so the
% spectrum is conjugate symmetric.  I have not bothered to use that
% property to reduce the number of points used to represent the
% spectrum; instead this function returns the entire, conjugate
% symmetric spectrum.

function spectra = analyze(input, N)

step = N/4;
Nspectra = 1 + (length(input) - N)/step ;
Nspectra = floor(Nspectra);

% Idea: Resynthesis is easy with Hanning window using a 4:1 overlap
% ratio, since all that needs to be done is adding 4 IFFT'd windows.
% But, at the edges there aren't 4 overlapped windows.  So we pad
% around the edges.  Then after resynthesis, we throw away the pad
% samples.
Npad = 3; 
spectra = zeros(Nspectra+2*Npad,N);

% Here, I duplicate a group of samples to pad the input.  In hindsight, it would be more
% straightforward simply to pad with zeros, and padding with 
% zeros would make the effect of the padding on the Fourier
% analysis more obvious. (Thanks to Birger Kollmeier for pointing
% this out.)  I have kept the duplication approach
% to keep consistency with my older experiments.  I did experiment
% with padding with zeros here and it made little difference
% to my results, compared to the duplication approach.
leftPad = input(1:step*Npad);
rightPad = input( length(input) - step*Npad + 1 : length(input));
input = [leftPad ; input ; rightPad];

h=hanning(N);
startIndex = 1;
for i=1:Nspectra+2*Npad
   endIndex = startIndex + N-1;
   windowedData = input(startIndex:endIndex) .* h;

   % .' = non-conjugate transpose 
   spectra(i,:)= fft(windowedData).';

   startIndex = startIndex + step;
end





	

