% Main routine for log spectral subtraction.
%
% Call this routine with the first argument set to a vector containing
% a time-domain audio signal.  The vector should be a column vector
% (i.e., a matrix with one column).  This routine returns a vector
% containing the processed time-domain output.
%
% If the "phase" argument to this routine is 1, this will do LTLSS
% using minimum-phase reconstruction to create a phase spectrum for the mean.
% If the "phase" argument is 0, this routine will do LTLSS without the 
% minimum-phase reconstruction.
%
% I have only tested this code with values
% of winLength such that the number of points
% in the FFT (equal to Fs * winLength) is a power of 2.
function r = phase_main(input, Fs, winLength, leftWing, rightWing, phase)

if (nargin==1)
  Fs = 8000; % Fs=sampling frequency in Hz
end


if (nargin<=2)
  winLength = 1.024;    % length of analysis window in seconds
                        % chose 1.024 to make N=power of 2  
end

%  leftWing and rightWing give the number of frames of past and future
%  to use in the mean calculation (clearer names might have been
%  leftContextLength and rightContextLength).  Calculating the mean
%  over ALL frames is also a viable option, and normalize.m could easily
%  be modified to support that.
if (nargin<=3)
  leftWing = 22; % number of frames of past to use
end

if (nargin<=4) 
  rightWing = 22; % number of frames of future to use
end

% Make sure there are an integer number of window steps in the sample
% by padding at the end if necessary.  
N=Fs*winLength;
step=N/4;
lengthOfTailPad = step - rem( length(input) - N, step );
tailPad = input( length(input) - lengthOfTailPad + 1 : length(input) );
input = [input ; tailPad ];

% Analysis.
spectra = analyze(input,N);

% Separation into phase and magnitude.
logmagspectra = log(abs(spectra));
phasespectra = angle(spectra);

% Normalization.
if (phase)
  logmagspectraN=phase_normalize(logmagspectra, leftWing, rightWing);
else
  logmagspectraN=normalize(logmagspectra, leftWing, rightWing);
end
  
% Combination of phase and magnitude.
spectraN = exp(logmagspectraN) .* exp(i*phasespectra);

% Resynthesis.
r = real ( resynth(spectraN) );  

% Throw away any tail padding we may have added at the beginning
r = r(1 : length(r) - lengthOfTailPad);

% Return a column vector.
r = transpose(r);

